<?php

namespace Bydedi\Lengow;

use League\Csv\Writer;
use Bydedi\Lengow\Marketplace;

/**
 * Class Lengow
 * @package Bydedi\Lengow\Lengow
 */
class Lengow
{
    const FORMAT_CSV = 1;
    const FORMAT_XML= 2;

    const STATUT_TOUS = 'all';
    const STATUT_NOUVEAU = 'new';
    const STATUT_EN_COURS = 'processing';
    const STATUT_EXPEDIE = 'shipped';
    const STATUT_ANNULE = 'cancel';

    /**
     * @var LengowInterface
     */
    protected $lengowSite;

    /**
     * @var Marketplace[]
     */
    protected $marketplaces;

    protected $sql;
    protected $boutique_id;
    protected $langue_id;
    protected $options;
    protected $callbacks;

    /**
     * @param mixed $boutique_id
     */
    public function setBoutiqueId($boutique_id)
    {
        $this->boutique_id = $boutique_id;
    }

    /**
     * @param mixed $langue_id
     */
    public function setLangueId($langue_id)
    {
        $this->langue_id = $langue_id;
    }

    /**
     * @param \mysqli $sql
     */
    public function setSql(\mysqli $sql)
    {
        $this->sql = $sql;
    }

    public function setOption($key, $value)
    {
        $this->options[$key] = $value;
    }

    public function getOption($key)
    {
        return array_key_exists($key, $this->options) ? $this->options[$key]: null;
    }

    public function setCallback($function, $key = null)
    {
        $this->callbacks[$key] = $function;
    }

    public function __construct(LengowInterface $lengowSite)
    {
        $this->marketplaces = $lengowSite->getMarketplaces();
        $this->lengowSite = $lengowSite;
        $this->callbacks = array();
        $this->options = array();
        $this->idsGroup = array(0);
    }

    private function loadDataMarketplace()
    {
        $xml = simplexml_load_file(__DIR__.'/marketplaces.xml');

        foreach ($this->marketplaces as $k => &$marketplace) {

            $name = $marketplace->getWsdl();
            $object = $xml->xpath('/marketplaces/marketplace[@name=\''.$name.'\']');

            if (empty($object)) {
                throw new \Exception('Marketplace non géré : ' . $name);
            }

            $object = $object[0];
            $marketplace->setApiUrl((string)$object->api);

            $states = array();

            foreach ($object->states->state as $state)
            {
                $aAction = array();

                if (isset($state->actions))
                {
                    foreach ($state->actions->action as $action)
                    {
                        $typeAction = (string)$action['type'];

                        $aParam = array();

                        $params = $xml->xpath('/marketplaces/marketplace[@name=\''.$name.'\']/additional_params/param[@usedby=\''.$typeAction.'\']');

                        if (count($params)) {
                            foreach ($params as $param) {

                                $typeParam = (string)$param->type;

                                $aParam[$typeParam] = array(
                                    'type' => $typeParam,
                                    'name' => (string)$param->name,
                                );

                                foreach ($param->attributes() as $key => $value) {
                                    $aParam[$typeParam][$key] = (string)$value;
                                }

                                if (isset($param->accepted_values)) {
                                    foreach ($param->accepted_values->value as $v) {
                                        $aParam[$typeParam]['accepted_values'][] = (string)$v;
                                    }

                                    $default = $xml->xpath('/marketplaces/marketplace[@name=\''.$name.'\']/additional_params/param[@usedby=\''.$typeAction.'\']/accepted_values/value[@default=\'true\']');

                                    if ($default) {
                                        $aParam[$typeParam]['accepted_values_default'] = (string)$default[0];
                                    }
                                }
                            }
                        }

                        $aAction[$typeAction] = array(
                            'type' => $typeAction,
                            'name' => (string)$action,
                            'params' => $aParam,
                        );
                    }
                }

                $states[(string)$state->lengow] = array(
                    'name' => (string)$state->lengow,
                    'actions' => $aAction,
                );
            }

            $marketplace->setStates($states);
        }
    }

    public function import()
    {
        $this->loadDataMarketplace($this->marketplaces);

        $dtDebut = !empty($this->options['date_debut']) && $this->options['date_debut'] instanceof \DateTime ? $this->options['date_debut']: new \DateTime();
        $dtFin = !empty($this->options['date_debut']) && $this->options['date_debut'] instanceof \DateTime ? $this->options['date_debut']: new \DateTime();

        $dDebut = $dtDebut->format('Y-m-d');
        $dFin = $dtFin->format('Y-m-d');

        foreach ($this->marketplaces as $k => $m) {

//            $url = 'http://api.lengow.com/V2/' . $dDebut . '/' . $dFin . '/' . $this->options['lengow_id'] . '/'.implode(',', $this->idsGroup).'/' . $m['flux_id'] . '/commands/all/newxml/';
            $url = 'http://api.lengow.com/V2/2015-02-27/2015-03-01/' . $this->options['lengow_id'] . '/'.implode(',', $this->lengowSite->getIdentifiantsGroupe()).'/' . $m['flux_id'] . '/commands/all/newxml/';

            $xml = simplexml_load_file($url);

            //http://api.lengow.com/V2/2016-01-14/2016-01-14/3485/0/91217/commands/all/newxml/
            //http://api.lengow.com/V2/#DATE_FROM#/#DATE_TO#/3485/#ID_GROUP#/#ID_FLUX#/commands/#STATE#/#FORMAT#/
            if ($xml->orders_count->count_total < 1) {
                continue;
            }

            foreach ($xml->orders->order as $commande) {

                if (false && $commande->order_status->lengow == self::STATUT_NOUVEAU) {
                    // on accepte la commande sur lengow
                    if (!empty($m['states'][self::STATUT_NOUVEAU]['actions']['accept']) && !empty($m['wsdl'])) {

                        $url = 'https://wsdl.lengow.com/wsdl/' . $m['wsdl'] . '/' . $m['flux_id'] . '/' . $commande->order_id . '/' . $m['states'][self::STATUT_NOUVEAU]['actions']['accept']['name'] . '.xml';
                        //file_get_contents($url);
                    }
                } elseif (true || $commande->order_status->lengow == self::STATUT_EN_COURS) {

                    // Vérification si commande expédiée par callback (doit renvoyer un tableau avec au moins l'id de la commande pour dire qu'elle existe)
                    $commandeLengow = $this->lengowSite->getCommandeExistante($commande->order_id);
//                    $aDataCommande = call_user_func($this->callbacks['check_commande'], $commande->order_id);

                    // $aDataCommande['expedition'] peut contenir les clefs 'carrier' (transporteur) et 'tracking' (numéro de suivi)
                    if ($commandeLengow->getStatus() == self::STATUT_EXPEDIE) {
                        if (!empty($m['states'][self::STATUT_EN_COURS]['actions']['shipped'])) {
                            // On signale à Lengow l'expédition
                            $url = 'https://wsdl.lengow.com/wsdl/' . $m['wsdl'] . '/' . $m['flux_id'] . '/' . $commande->order_id . '/' . $m['states'][self::STATUT_EN_COURS]['actions']['shipped']['name'] . '.xml';

                            $aParams = array();

                            $carrier = $commandeLengow->getCarrier();
                            $tracking = $commandeLengow->getTracking();

                            if (!empty($carrier) && !empty($m['states'][self::STATUT_EN_COURS]['actions']['shipped']['params']['carrier'])) {
                                $aParams[] = $m['states'][self::STATUT_EN_COURS]['actions']['shipped']['params']['carrier']['name'].'='.$carrier;
                            }

                            if (!empty($tracking) && !empty($m['states'][self::STATUT_EN_COURS]['actions']['shipped']['params']['tracking'])) {
                                $aParams[] = $m['states'][self::STATUT_EN_COURS]['actions']['shipped']['params']['tracking']['name'].'='.$tracking;
                            }

                            if (!empty($aParams)) {
                                $url .= '?'.implode('&', $aParams);
                            }

                            //file_get_contents($url);
                        }

                    } elseif (empty($aDataCommande)) {

                        // Récupération des données de la commande pour création
                        $aData = array(
                            'commande' => array(
                                'client_id' => $m['client_id'],
                                'paiement_id' => $m['paiement_id'],
                                'livraison_id' => $m['livraison_id'],
                                'marketplace' => (string)$commande->marketplace,
                                'flux_id' => (string)$commande->idFlux,
                                'statut_lengow' => (string)$commande->lengow,
                                'commande_referer' => (string)$commande->order_id,
                                'commande_date_ajout' => (string)$commande->order_purchase_date . ' ' . (string)$commande->order_purchase_heure,
                                'commande_date_modif' => (string)$commande->order_purchase_date . ' ' . (string)$commande->order_purchase_heure,
                                'taxe_taux' => (string)$commande->order_tax,
                                'commande_total_ttc' => (string)$commande->order_amount,
                                'commande_port_ttc' => (string)$commande->order_shipping,
                                'commission' => (string)$commande->order_commission,
                                'frais_traitement' => (string)$commande->order_processing_fee,
                                'devise' => (string)$commande->order_currency,
                            ),
                            'paiement' => array(
                                'checkout' => (string)$commande->order_payment->payment_checkout,
                                'statut' => (string)$commande->order_payment->payment_status,
                                'type' => (string)$commande->order_payment->payment_type,
                                'date' => (string)$commande->order_payment->payment_date . ' ' . (string)$commande->order_payment->payment_heure,
                            ),
                            'facture' => array(
                                'numero' => (string)$commande->order_invoice->invoice_number,
                                'url' => (string)$commande->order_invoice->invoice_url,
                            ),
                            'facturation' => array(
                                'civilite' => (string)$commande->billing_address->billing_civility,
                                'societe' => (string)$commande->billing_address->billing_society,
                                'nom' => (string)$commande->billing_address->billing_lastname,
                                'prenom' => (string)$commande->billing_address->billing_firstname,
                                'email' => (string)$commande->billing_address->billing_email,
                                'adresse1' => (string)$commande->billing_address->billing_address,
                                'adresse2' => (string)$commande->billing_address->billing_address_2,
                                'code_postal' => (string)$commande->billing_address->billing_zipcode,
                                'ville' => (string)$commande->billing_address->billing_city,
                                'pays' => (string)$commande->billing_address->billing_country,
                                'pays_iso' => (string)$commande->billing_address->billing_country_iso,
                                'telephone' => (string)$commande->billing_address->billing_phone_home,
                                'telephone_bureau' => (string)$commande->billing_address->billing_phone_office,
                                'telephone_mobile' => (string)$commande->billing_address->billing_phone_mobile,
                                'adresse_complete' => (string)$commande->billing_address->billing_full_address,
                            ),
                            'livraison' => array(
                                'civilite' => (string)$commande->delivery_address->delivery_civility,
                                'societe' => (string)$commande->delivery_address->delivery_society,
                                'nom' => (string)$commande->delivery_address->delivery_lastname,
                                'prenom' => (string)$commande->delivery_address->delivery_firstname,
                                'email' => (string)$commande->delivery_address->delivery_email,
                                'adresse1' => (string)$commande->delivery_address->delivery_address,
                                'adresse2' => (string)$commande->delivery_address->delivery_address_2,
                                'code_postal' => (string)$commande->delivery_address->delivery_zipcode,
                                'ville' => (string)$commande->delivery_address->delivery_city,
                                'pays' => (string)$commande->delivery_address->delivery_country,
                                'pays_iso' => (string)$commande->delivery_address->delivery_country_iso,
                                'telephone' => (string)$commande->delivery_address->delivery_phone_home,
                                'telephone_bureau' => (string)$commande->delivery_address->delivery_phone_office,
                                'telephone_mobile' => (string)$commande->delivery_address->delivery_phone_mobile,
                                'adresse_complete' => (string)$commande->delivery_address->delivery_full_address,
                            ),
                            'tracking' => array(
                                'methode' => (string)$commande->tracking_informations->tracking_method,
                                'transporteur' => (string)$commande->tracking_informations->tracking_carrier,
                                'numero_colis' => (string)$commande->tracking_informations->tracking_number,
                                'url_suivi' => (string)$commande->tracking_informations->tracking_url,
                                'date_expedition' => (string)$commande->tracking_informations->tracking_shipped_date,
                                'numero_point_relai' => (string)$commande->tracking_informations->tracking_relay,
                                'gestion_par_marketplace' => (string)$commande->tracking_informations->tracking_deliveringByMarketPlace,
                            ),
                            'divers' => array(
                                'commentaire' => (string)$commande->order_comments,
                                'id_client_marketplace' => (string)$commande->customer_id,
                                'ip_client' => (string)$commande->order_ip,
                                'quantite_produits' => (string)$commande->order_items,
                                'nombre_produits' => (string)$commande->cart->cart_products,
                            ),
                        );

                        $aData = $this->transformationRelais($aData);

                        $aProduit = array();

                        foreach ($commande->cart->products->product as $item) {
                            $aProduit[] = array(
                                'id_lengow' => (string)$item->idLengow,
                                'id_marketplace' => (string)$item->idMP,
                                'sku' => (string)$item->sku,
                                'ean' => (string)$item->ean,
                                'nom' => (string)$item->title,
                                'categorie' => (string)$item->category,
                                'marque' => (string)$item->brand,
                                'url' => (string)$item->url_product,
                                'image' => (string)$item->url_image,
                                'id_ligne_marketplace' => (string)$item->order_lineid,
                                'quantite' => (string)$item->quantity,
                                'prix_ttc' => (string)$item->price,
                                'prix_ttc_unitaire' => (string)$item->price_unit,
                                'frais_port' => (string)$item->shipping_price,
                                'taxe' => (string)$item->tax,
                                'statut' => (string)$item->status,
                            );
                        }

                        $aData['produits'] = $aProduit;

                        // On renvoie la commande dans le projet pour création
                        $commande_id = $this->lengowSite->creerCommande($aData);
                    }
                }
            }
        }
    }

    private function getRequeteExport($aExport)
    {
        $sql = "SELECT ";

        foreach (array_merge($aExport['SELECT'], $aExport['SELECT_TEMP']) as $k => $v) {
            if (!empty($v)) {
                $sql .= $v . " AS " . $k . ', ';
            }
        }

        $sql = trim($sql, ', ') . ' FROM ' . $aExport['FROM'];

        foreach ($aExport['INNER JOIN'] as $k => $v) {
            if (!empty($v)) {
                $sql .= ' INNER JOIN ' . $v . ' ';
            }
        }

        foreach ($aExport['LEFT JOIN'] as $k => $v) {
            if (!empty($v)) {
                $sql .= ' LEFT JOIN ' . $v . ' ';
            }
        }

        $sql .= ' WHERE 1 ';

        foreach ($aExport['WHERE'] as $k => $v) {
            $sql .= ' AND ' . $v;
        }

        $sql .= " GROUP BY p.produit_id, pa.produit_attribut_id";

        return $sql;
    }

    private function getDefaultExport()
    {
        return array(
            "SELECT" => array(
                "IDPere" => "p.produit_id",
                "Identifiant_unique" => "IF (pa.produit_attribut_id, CONCAT(p.produit_id, '_', pa.produit_attribut_id), p.produit_id)",
                "Titre" => "pn.produit_nom_nom",
                "Description" => "pn.produit_nom_desc",
                "Marque" => "m.marque_nom",
                "Devise" => "'EUR'",
                "Categorie" => "IF(c3.categorie_id, cn3.categorie_nom_nom, IF(c2.categorie_id, cn2.categorie_nom_nom, cn1.categorie_nom_nom))",
                "Sous_Categorie1" => "IF(c3.categorie_id, cn2.categorie_nom_nom, cn1.categorie_nom_nom)",
                "Sous_Categorie2" => "IF(c3.categorie_id, cn1.categorie_nom_nom, NULL)",
                "Prix_TTC" => "ROUND((produit_prix + COALESCE(produit_attribut_prix, 0)) * (1 + t.taxe_taux), 2)",
                "Prix_barre" => "ROUND(ppb.produit_prix_barre_prix * (1 + t.taxe_taux), 2)",
                "Date_debut_promo" => "ppb.produit_prix_barre_date_debut",
                "Date_fin_promo" => "ppb.produit_prix_barre_date_fin",
                "URL_produit" => "",
                "URL_image" => "COALESCE(pi.produit_image, '')",
                "Occasion" => "'0'",
                "Shopinfo" => "b.boutique_nom",
                "Quantite_en_stock" => "IF(pa.produit_attribut_id, pa.produit_attribut_quantite_p, p.produit_quantite_p)",
                "Disponibilite" => "'0'",
                "Delais_de_livraison" => "'2'",
                "Description_de_livraison" => "'socolissimo'",
                "MPN" => "IF(pa.produit_attribut_id, COALESCE(pa.produit_attribut_ref, p.produit_ref), p.produit_ref)",
                "Libelle_fournisseur" => "''",
                "EAN" => "IF(pa.produit_attribut_id, pa.produit_attribut_ean, p.produit_ean)",
                "Poids" => "",
                "Garantie" => "",
                "Bundle" => "",
                "D3E" => "0",
                "Frais_de_livraison" => "'8'",
            ),
            "FROM" => "produit p",
            "INNER JOIN" => array(
                "b" => "boutique b ON b.boutique_id = '" . $this->boutique_id . "'",
                "t" => "taxe t ON t.taxe_id = p.taxe_id",
            ),
            "LEFT JOIN" => array(
                "pa" => "produit_attribut pa ON pa.produit_id = p.produit_id",
                "pn" => "produit_nom pn ON pn.produit_id = p.produit_id AND pn.langue_id = ".$this->langue_id,
                "m" => "marque m ON m.marque_id = p.produit_id",
                "ppb" => "(
                    SELECT *
                    FROM produit_prix_barre
                    WHERE produit_prix_barre_date_debut <= NOW()
                    AND produit_prix_barre_date_fin >= NOW()
                    AND boutique_id = ".$this->boutique_id."
                    GROUP BY produit_id, produit_attribut_id
                    ORDER BY produit_prix_barre_valeur ASC
                ) AS ppb ON ppb.produit_id = p.produit_id AND ppb.produit_attribut_id = pa.produit_attribut_id",
                "pi" => "produit_image pi ON pi.produit_id = p.produit_id AND pi.produit_image_default = 1",
                "pc" => "(
                    SELECT produit_id, categorie_id
                    FROM produit_categorie
                    GROUP BY produit_id
                ) AS pc ON pc.produit_id = p.produit_id",
                "c1" => "categorie c1 ON c1.categorie_id = pc.categorie_id",
                "c2" => "categorie c2 ON c2.categorie_id = c1.parent_id",
                "c3" => "categorie c3 ON c3.categorie_id = c2.parent_id",
                "cn1" => "categorie_nom cn1 ON cn1.categorie_id = c1.categorie_id AND cn1.langue_id = ".$this->langue_id,
                "cn2" => "categorie_nom cn2 ON cn2.categorie_id = c2.categorie_id AND cn2.langue_id = ".$this->langue_id,
                "cn3" => "categorie_nom cn3 ON cn3.categorie_id = c3.categorie_id AND cn3.langue_id = ".$this->langue_id,
            ),
            "SELECT_TEMP" => array(),
        );
    }

    public function export($format)
    {
        if (empty($format)) {
            throw new \Exception('Paramètre format requis.');
        }

        $aExportOk = $this->getDefaultExport();

        if (!empty($this->options['champs_export']) && is_array($this->options['champs_export'])) {
            $aExportOk = array_replace_recursive($aExportOk, $this->options['champs_export']);
        }

        $sql = $this->getRequeteExport($aExportOk);

        $result = $this->sql->query($sql);

        $aData = array();

        while ($row = $result->fetch_array(MYSQLI_ASSOC)) {
            $aData[] = $row;
        }

        // Traitements supplémentaires
        foreach ($aData as &$v) {

            foreach ($this->callbacks as $f) {
                $f($v);
            }

            // On supprime les clefs temporaires
            $v = array_diff_key($v, $aExportOk['SELECT_TEMP']);
        }

        switch ($format) {

            case self::FORMAT_CSV:

                if (!empty($this->options['nom_fichier'])) {
                    if (!is_file($this->options['nom_fichier'])) {
                        touch($this->options['nom_fichier']);
                    }
                    $fileObject = new \SplFileObject($this->options['nom_fichier']);
                } else {
                    $fileObject = new \SplTempFileObject();
                }

                $csv = Writer::createFromFileObject($fileObject);
                $csv->setDelimiter(';');
                $csv->setOutputBOM(Writer::BOM_UTF8);

                $csv->insertOne(array_keys($aExportOk['SELECT']));
                $csv->insertAll($aData);

                if (empty($this->options['nom_fichier'])) {
                    $csv->output('catalogue.csv');
                    exit;
                }

                break;

            case self::FORMAT_XML:

                $topNode = array_key_exists('topNode', $this->options) ? $this->options['topNode']: 'catalog';
                $productNode = array_key_exists('productNode', $this->options) ? $this->options['productNode']: 'product';

                $xmlData = new \SimpleXMLElement('<?xml version="1.0"?><'.$topNode.'></'.$topNode.'>');

                $xmlNode = function ($data, &$xmlData) use (&$xmlNode, $productNode) {
                    foreach ($data as $key => $value) {
                        if (is_array($value)) {
                            if (is_numeric($key)) {
                                $key = $productNode;
                            }
                            $subnode = $xmlData->addChild($key);
                            $xmlNode($value, $subnode);
                        } else {
                            $xmlData->addChild("$key",htmlspecialchars("$value"));
                        }
                    }
                };

                $xmlNode($aData, $xmlData);

                if (empty($this->options['nom_fichier'])) {
                    header('Content-Type: text/xml');
                    echo $xmlData->asXML();
                    exit;
                } else {
                    $xmlData->asXML($this->options['nom_fichier']);
                }

                break;
            default:
                throw new \Exception('Format non géré : ' . $format);
                break;
        }
    }

    /**
     * Ecrase l'adresse de livraison si livraison en relais
     * @param $data
     */
    private function transformationRelais(&$data)
    {
        $transporteur = $data['tracking']['transporteur'];
        $flux_id = $data['commande']['flux_id'];

        foreach ($this->marketplaces as $m) {
            if ($m->getFluxId() == $flux_id) {
                foreach ($m->getLivraisons() as $l) {
                    if ($l->getTransporteur() == $transporteur && $l->isRelais()) {
                        $data = $this->lengowSite->setAdresseRelais($data);
                        break;
                    }
                }
            }
        }
    }
}
